@@ -10,24 +10,30 @@ import tempfile
 import traceback
 import uuid
 from collections import OrderedDict
+from io import IOBase
+from types import TracebackType
 from typing import (
-    IO,
     Any,
     Callable,
     Dict,
+    IO,
     Iterable,
+    Iterator,
     List,
     MutableMapping,
     Optional,
     Set,
+    Type,
+    TypeVar,
     Union,
 )
 from urllib.parse import urlparse
 
 import portalocker  # type: ignore
 from iopath.common.download import download
-from iopath.common.event_logger import VTYPE, EventLogger
+from iopath.common.event_logger import EventLogger, VTYPE
 from iopath.common.non_blocking_io import NonBlockingIOManager
+from typing_extensions import Protocol
 
 
 __all__ = ["LazyPath", "PathManager", "get_cache_dir", "file_lock"]
@@ -92,6 +98,7 @@ def file_lock(path: str):  # type: ignore
     return portalocker.Lock(path + ".lock", timeout=3600)  # type: ignore
 
 
+# pyre-fixme[24]: Generic type `os.PathLike` expects 1 type parameter.
 class LazyPath(os.PathLike):
     """
     A path that's lazily evaluated when it's used.
@@ -126,6 +133,7 @@ class LazyPath(os.PathLike):
     # behave more like a str after evaluated
     def __getattr__(self, name: str):  # type: ignore
         if name in LazyPath.KEPT_ATTRIBUTES:
+            # pyre-fixme[16]: `PathLike` has no attribute `__getattr__`.
             return super().__getattr__(name)
         if self._value is None:
             raise AttributeError(f"Uninitialized LazyPath has no attribute: {name}.")
@@ -143,6 +151,27 @@ class LazyPath(os.PathLike):
             return super().__str__()
 
 
+class TabularIO(Protocol):
+    """
+    Context Manager interface to be used by PathHandler methods.
+    """
+
+    def __enter__(self) -> "TabularIO":
+        ...
+
+    def __exit__(
+        self,
+        exc_type: Optional[Type[BaseException]],
+        exc_val: Optional[BaseException],
+        exc_tb: Optional[TracebackType],
+    ) -> None:
+        ...
+
+    # pyre-fixme[3]: Return annotation cannot contain `Any`.
+    def __iter__(self) -> Iterator[Any]:
+        ...
+
+
 class PathHandler(EventLogger):
     """
     PathHandler is a base class that defines common I/O functionality for a URI
@@ -171,6 +200,7 @@ class PathHandler(EventLogger):
                 ```
         """
         super().__init__()
+        # pyre-fixme[4]: Attribute must be annotated.
         self._non_blocking_io_manager = None
         self._non_blocking_io_executor = async_executor
         try:
@@ -244,7 +274,7 @@ class PathHandler(EventLogger):
 
     def _opent(
         self, path: str, mode: str = "r", buffering: int = 32, **kwargs: Any
-    ) -> Iterable[Any]:
+    ) -> TabularIO:
         raise NotImplementedError()
 
     def _open(
@@ -275,7 +305,7 @@ class PathHandler(EventLogger):
         callback_after_file_close: Optional[Callable[[None], None]] = None,
         buffering: int = -1,
         **kwargs: Any,
-    ) -> Union[IO[str], IO[bytes]]:
+    ) -> IOBase:
         """
         Open a file with asynchronous methods. `f.write()` calls will be dispatched
         asynchronously such that the main program can continue running.
@@ -323,7 +353,7 @@ class PathHandler(EventLogger):
         # Restrict mode until `NonBlockingIO` has async read feature.
         valid_modes = {"w", "a", "b"}
         if not all(m in valid_modes for m in mode):
-            raise ValueError("`opena` mode must be write or append")
+            raise ValueError(f"`opena` mode must be write or append for path {path}")
 
         # TODO: Each `PathHandler` should set its own `self._buffered`
         # parameter and pass that in here. Until then, we assume no
@@ -352,6 +382,8 @@ class PathHandler(EventLogger):
                 "is most likely due to invalid `opena` args. Make sure "
                 "they match the `open` args for the `PathHandler`."
             )
+            # pyre-fixme[7]: Expected `Union[IO[bytes], IO[str]]` but got implicit
+            #  return value of `None`.
             self._async_close()
 
     def _async_join(self, path: Optional[str] = None, **kwargs: Any) -> bool:
@@ -534,6 +566,7 @@ class NativePathHandler(PathHandler):
     handler uses `open()` and `os.*` calls on the given path.
     """
 
+    # pyre-fixme[4]: Attribute must be annotated.
     _cwd = None
 
     def __init__(
@@ -565,6 +598,7 @@ class NativePathHandler(PathHandler):
         errors: Optional[str] = None,
         newline: Optional[str] = None,
         closefd: bool = True,
+        # pyre-fixme[24]: Generic type `Callable` expects 2 type parameters.
         opener: Optional[Callable] = None,
         **kwargs: Any,
     ) -> Union[IO[str], IO[bytes]]:
@@ -862,6 +896,8 @@ class OneDrivePathHandler(HTTPURLHandler):
     def _get_supported_prefixes(self) -> List[str]:
         return [self.ONE_DRIVE_PREFIX]
 
+    # pyre-fixme[14]: `_get_local_path` overrides method defined in `HTTPURLHandler`
+    #  inconsistently.
     def _get_local_path(self, path: str, force: bool = False, **kwargs: Any) -> str:
         """
         This implementation downloads the remote resource and caches it locally.
@@ -913,6 +949,7 @@ class PathManager:
         Flag for enabling / disabling telemetry.
         """
 
+    # pyre-fixme[24]: Generic type `os.PathLike` expects 1 type parameter.
     def __get_path_handler(self, path: Union[str, os.PathLike]) -> PathHandler:
         """
         Finds a PathHandler that supports the given path. Falls back to the native
@@ -952,6 +989,8 @@ class PathManager:
                 )
                 self._enable_logging = False
 
+    # pyre-fixme[34]: `Variable[VTYPE <: [str, int, bool, float]]` isn't present in
+    #  the function's parameters.
     def __get_open_keys(self, path: str, mode: str, buffering: int) -> Dict[str, VTYPE]:
         """
         Helper function to return common set of key-value pairs applicable to open apis.
@@ -981,7 +1020,7 @@ class PathManager:
 
     def opent(
         self, path: str, mode: str = "r", buffering: int = 32, **kwargs: Any
-    ) -> Iterable[Any]:
+    ) -> TabularIO:
         """
         Open a tabular data source. Only reading is supported.
         The opent() returns a Python iterable collection object, compared to bytes/text data with open()
@@ -993,7 +1032,7 @@ class PathManager:
             buffering (int): number of rows fetched and cached
 
         Returns:
-            An iterable collection object.
+            A TabularIO context manager object
         """
         return self.__get_path_handler(path)._opent(path, mode, buffering, **kwargs)
 
@@ -1033,7 +1072,7 @@ class PathManager:
         buffering: int = -1,
         callback_after_file_close: Optional[Callable[[None], None]] = None,
         **kwargs: Any,
-    ) -> Union[IO[str], IO[bytes]]:
+    ) -> IOBase:
         """
         Open a file with asynchronous methods. `f.write()` calls will be dispatched
         asynchronously such that the main program can continue running.
@@ -1213,6 +1252,8 @@ class PathManager:
         except TypeError:
             bret = handler._get_local_path(path, **kwargs)
         kvs = {"op": "get_local_path", "path": path, "force": force}
+        # pyre-fixme[6]: For 2nd param expected `Dict[str, Variable[VTYPE <: [str,
+        #  int, bool, float]]]` but got `Dict[str, Union[bool, str]]`.
         self.__log_tmetry_keys(handler, kvs)
         return bret
 
@@ -1233,7 +1274,7 @@ class PathManager:
         Returns:
             status (bool): True on success
         """
-        assert os.path.exists(local_path)
+        assert os.path.exists(local_path), f"local_path = {local_path}"
         handler = self.__get_path_handler(dst_path)
 
         kvs = {
@@ -1245,7 +1286,10 @@ class PathManager:
         bret = handler._copy_from_local(
             local_path=local_path, dst_path=dst_path, overwrite=overwrite, **kwargs
         )
+        # pyre-fixme[6]: For 2nd param expected `Dict[str, Variable[VTYPE <: [str,
+        #  int, bool, float]]]` but got `Dict[str, Union[bool, str]]`.
         self.__log_tmetry_keys(handler, kvs)
+        # pyre-fixme[7]: Expected `bool` but got `None`.
         return bret
 
     def exists(self, path: str, **kwargs: Any) -> bool:
@@ -1367,6 +1411,8 @@ class PathManager:
         """
         if path is None and self._cwd is None:
             return True
+        # pyre-fixme[6]: For 1st param expected `Union[PathLike[typing.Any], str]`
+        #  but got `Optional[str]`.
         handler = self.__get_path_handler(path or self._cwd)
         if self.__get_path_handler(path or self._cwd)._set_cwd(path, **kwargs):  # type: ignore
             self._cwd = path
@@ -1374,6 +1420,8 @@ class PathManager:
         else:
             bret = False
         kvs = {"op": "set_cwd", "path": path}
+        # pyre-fixme[6]: For 2nd param expected `Dict[str, Variable[VTYPE <: [str,
+        #  int, bool, float]]]` but got `Dict[str, Optional[str]]`.
         self.__log_tmetry_keys(handler, kvs)
         return bret
 
@@ -1463,6 +1511,8 @@ class PathManager:
         for handler in self._path_handlers.values():
             handler._strict_kwargs_check = enable
 
+    # pyre-fixme[3]: Return type must be annotated.
+    # pyre-fixme[2]: Parameter must be annotated.
     def set_logging(self, enable_logging=True):
         self._enable_logging = enable_logging
 
@@ -1475,6 +1525,7 @@ class PathManager:
         assert dst_handler._copy_from_local is not None
 
         local_file = src_handler._get_local_path(src_path, **kwargs)
+        # pyre-fixme[7]: Expected `bool` but got `None`.
         return dst_handler._copy_from_local(
             local_file, dst_path, overwrite=overwrite, **kwargs
         )
@@ -1511,9 +1562,11 @@ class PathManagerFactory:
     """
 
     GLOBAL_PATH_MANAGER = "global_path_manager"
+    # pyre-fixme[4]: Attribute must be annotated.
     pm_list = {}
 
     @staticmethod
+    # pyre-fixme[2]: Parameter must be annotated.
     def get(key=GLOBAL_PATH_MANAGER, defaults_setup=False) -> PathManager:
         """
         Get the path manager instance associated with a key.
@@ -1536,6 +1589,8 @@ class PathManagerFactory:
         return PathManagerFactory.pm_list[key]
 
     @staticmethod
+    # pyre-fixme[3]: Return type must be annotated.
+    # pyre-fixme[2]: Parameter must be annotated.
     def remove(key):
         """
         Remove the path manager instance associated with a key.
@@ -1552,4 +1607,5 @@ A global instance of PathManager.
 This global instance is provided for backward compatibility, but it is
 recommended that clients use PathManagerFactory
 """
+# pyre-fixme[5]:  Global expression must be annotated.
 g_pathmgr = PathManagerFactory.get(defaults_setup=True)
